@@ -16,7 +16,7 @@
 # along with this library; if not, write to the Free Software Foundation, Inc.,
 # 59 Temple Place - Suite 330, Boston, MA  02111-1307  USA.
 #
-# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Gtk2/Gtk2.pm,v 1.102 2007/01/08 19:48:06 kaffeetisch Exp $
+# $Header: /cvsroot/gtk2-perl/gtk2-perl-xs/Gtk2/Gtk2.pm,v 1.107 2007/07/08 19:23:58 kaffeetisch Exp $
 #
 
 package Gtk2;
@@ -35,7 +35,7 @@ eval "use Cairo;";
 
 require DynaLoader;
 
-our $VERSION = '1.151';
+our $VERSION = '1.153';
 
 our @ISA = qw(DynaLoader);
 
@@ -102,8 +102,98 @@ use overload
 	'&{}' => sub { \&Gtk2::TreeSortable::IterCompareFunc::invoke },
 	fallback => 1;
 
-package Gtk2;
+package Gtk2::TreeModelSort;
 
+# We forgot to prepend Gtk2::TreeModel to @Gtk2::TreeModelSort::ISA.  So this
+# hack is here to make sure that $model_sort->get resolves to
+# Gtk2::TreeModel::get when appropriate and to Glib::Object::get otherwise, so
+# we stay backwards compatible.
+sub get {
+	if (@_ > 1 and ref $_[1] eq 'Gtk2::TreeIter') {
+		# called as $model->get ($iter, columns...);
+		return Gtk2::TreeModel::get (@_);
+	} else {
+		# called as $model->get (names...);
+		return Glib::Object::get (@_);
+	}
+}
+
+package Gtk2::Builder;
+
+sub _do_connect {
+  my ($object,
+      $signal_name,
+      $user_data,
+      $connect_object,
+      $flags,
+      $handler) = @_;
+
+  my $func = $flags & qw/after/ ? 'signal_connect_after' : 'signal_connect';
+
+  # we get connect_object when we're supposed to call
+  # signal_connect_object, which ensures that the data (an object)
+  # lives as long as the signal is connected.  the bindings take
+  # care of that for us in all cases, so we only have signal_connect.
+  # if we get a connect_object, just use that instead of user_data.
+  $object->$func($signal_name => $handler,
+                 $connect_object ? $connect_object : $user_data);
+}
+
+sub connect_signals {
+  my $builder = shift;
+  my $user_data = shift;
+
+  # $builder->connect_signals ($user_data)
+  # $builder->connect_signals ($user_data, $package)
+  if ($#_ <= 0) {
+    my $package = shift;
+    $package = caller unless defined $package;
+
+    $builder->connect_signals_full(sub {
+      my ($builder,
+          $object,
+          $signal_name,
+          $handler_name,
+          $connect_object,
+          $flags) = @_;
+
+      no strict qw/refs/;
+
+      my $handler = $handler_name;
+      if (ref $package) {
+        $handler = sub { $package->$handler_name(@_) };
+      } else {
+        if ($package && $handler !~ /::/) {
+          $handler = $package.'::'.$handler_name;
+        }
+      }
+
+      _do_connect ($object, $signal_name, $user_data, $connect_object,
+                   $flags, $handler);
+    });
+  }
+
+  # $builder->connect_signals ($user_data, %handlers)
+  else {
+    my %handlers = @_;
+
+    $builder->connect_signals_full(sub {
+      my ($builder,
+          $object,
+          $signal_name,
+          $handler_name,
+          $connect_object,
+          $flags) = @_;
+
+      return unless exists $handlers{$handler_name};
+
+      _do_connect ($object, $signal_name, $user_data, $connect_object,
+                   $flags, $handlers{$handler_name});
+    });
+  }
+}
+
+package Gtk2;
 
 1;
 __END__
